# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DocumentCountParams"]


class DocumentCountParams(TypedDict, total=False):
    classification_facet: Annotated[List[str], PropertyInfo(alias="classificationFacet")]
    """
    **Classification facet**: Filters results based on hierarchical categories. Each
    element in the list represents a level in the hierarchy:

    - `["Staatsrecht", "Grundrecht"]`: filters to Staatsrecht (Public Law) and
      Grundrecht (Fundamental Rights).
    - `["Privatrecht", "Zivilrecht", "Vertragsrecht"]`: filters to Privatrecht
      (Private Law), Zivilrecht (Civil Law), and Vertragsrecht (Contract Law).
    - `["Strafrecht", "Straftaten"]`: filters to Strafrecht (Criminal Law) and
      Straftaten (Offenses).

    See the full classification hierarchy:

    ```plaintext
    Staatsrecht
    ├── Abgaberecht & öffentliche Finanzen
    ├── Bau- & Planungsrecht
    ├── Bürger- & Ausländerrecht
    ├── Energie/Verkehr/Medien
    │   └── Verkehr
    ├── Grundrecht
    ├── Gesundheit & soziale Sicherheit
    ├── Öffentliches Dienstverhältnis
    ├── Ökologisches Gleichgewicht
    ├── Politische Rechte
    ├── Rechtshilfe & Auslieferung
    ├── Schuldbetreibungs- & Konkursrecht
    ├── Sozialversicherungsrecht
    │   ├── AHV
    │   ├── ALV
    │   ├── BV
    │   ├── EL
    │   ├── IV
    │   ├── KV
    │   └── UV
    ├── Unterrichtswesen & Berufsausbildung
    ├── Verfahren
    │   ├── Strafprozess
    │   ├── Zivilprozess
    │   └── Zuständigkeitsfragen
    └── Verfahrensrecht

    Privatrecht
    ├── Immaterialgüter-, Wettbewerbs- & Kartellrecht
    ├── Obligationen- & Handelsrecht
    │   ├── Gesellschaftsrecht
    │   ├── Haftpflichtrecht
    │   ├── Obligationsrecht (allgemein)
    │   └── Vertragsrecht
    └── Zivilrecht
       ├── Erbrecht
       ├── Familienrecht
       ├── Personenrecht
       └── Sachenrecht

    Strafrecht
    ├── Straf- & Massnahmenvollzug
    ├── Straftaten
    └── Strafrecht (allgemein)
    ```
    """

    query: str
    """**Search query**: Retrieves the count of documents matching the criteria.

    - Terms are **implicitly ANDed** (e.g., `hund biss` only matches documents
      containing both terms).
    - Supports **exact phrases** (e.g., `"hund spazieren"`) and **proximity
      searches** (e.g., `"hund biss"~10`).
    - **Excludes terms** with `-term` (e.g., `beschattung -besonnung`).
    - **Prefix search** with `term*` for terms with at least 3 characters.
    - **Synonym expansion and translations** with lower relevance ranking.

    For advanced operators and tips, see
    [Justement Search Tips](https://justement.ch/en/search).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DocumentByRefParams"]


class DocumentByRefParams(TypedDict, total=False):
    doc_ref: Required[Annotated[str, PropertyInfo(alias="docRef")]]
    """The legal reference of the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .snippet import (
    SnippetResource,
    AsyncSnippetResource,
    SnippetResourceWithRawResponse,
    AsyncSnippetResourceWithRawResponse,
    SnippetResourceWithStreamingResponse,
    AsyncSnippetResourceWithStreamingResponse,
)
from .document import (
    DocumentResource,
    AsyncDocumentResource,
    DocumentResourceWithRawResponse,
    AsyncDocumentResourceWithRawResponse,
    DocumentResourceWithStreamingResponse,
    AsyncDocumentResourceWithStreamingResponse,
)

__all__ = [
    "DocumentResource",
    "AsyncDocumentResource",
    "DocumentResourceWithRawResponse",
    "AsyncDocumentResourceWithRawResponse",
    "DocumentResourceWithStreamingResponse",
    "AsyncDocumentResourceWithStreamingResponse",
    "SnippetResource",
    "AsyncSnippetResource",
    "SnippetResourceWithRawResponse",
    "AsyncSnippetResourceWithRawResponse",
    "SnippetResourceWithStreamingResponse",
    "AsyncSnippetResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import snippet_document_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.document import Document

__all__ = ["SnippetResource", "AsyncSnippetResource"]


class SnippetResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SnippetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/justement-api/justement-python#accessing-raw-response-data-eg-headers
        """
        return SnippetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnippetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/justement-api/justement-python#with_streaming_response
        """
        return SnippetResourceWithStreamingResponse(self)

    def document(
        self,
        *,
        doc_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document by its document identifier.

        The docId is returned by the `search` endpoint as part of the result snippets.
        The response includes the full document content and metadata.

        Args:
          doc_id: The `docId` of the document that should be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/document",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"doc_id": doc_id}, snippet_document_params.SnippetDocumentParams),
            ),
            cast_to=Document,
        )


class AsyncSnippetResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSnippetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/justement-api/justement-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSnippetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnippetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/justement-api/justement-python#with_streaming_response
        """
        return AsyncSnippetResourceWithStreamingResponse(self)

    async def document(
        self,
        *,
        doc_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document by its document identifier.

        The docId is returned by the `search` endpoint as part of the result snippets.
        The response includes the full document content and metadata.

        Args:
          doc_id: The `docId` of the document that should be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/document",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"doc_id": doc_id}, snippet_document_params.SnippetDocumentParams),
            ),
            cast_to=Document,
        )


class SnippetResourceWithRawResponse:
    def __init__(self, snippet: SnippetResource) -> None:
        self._snippet = snippet

        self.document = to_raw_response_wrapper(
            snippet.document,
        )


class AsyncSnippetResourceWithRawResponse:
    def __init__(self, snippet: AsyncSnippetResource) -> None:
        self._snippet = snippet

        self.document = async_to_raw_response_wrapper(
            snippet.document,
        )


class SnippetResourceWithStreamingResponse:
    def __init__(self, snippet: SnippetResource) -> None:
        self._snippet = snippet

        self.document = to_streamed_response_wrapper(
            snippet.document,
        )


class AsyncSnippetResourceWithStreamingResponse:
    def __init__(self, snippet: AsyncSnippetResource) -> None:
        self._snippet = snippet

        self.document = async_to_streamed_response_wrapper(
            snippet.document,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Document"]


class Document(BaseModel):
    doc_id: str = FieldInfo(alias="docId")

    language: str

    name: str

    organ: str

    text: str

    url: str

    year: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    Language,
    document_by_id_params,
    document_count_params,
    document_by_ref_params,
    document_search_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncJustementPagination, AsyncJustementPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.snippet import Snippet
from ..types.document import Document
from ..types.language import Language
from ..types.document_count_response import DocumentCountResponse

__all__ = ["DocumentResource", "AsyncDocumentResource"]


class DocumentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DocumentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/justement-api/justement-python#accessing-raw-response-data-eg-headers
        """
        return DocumentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/justement-api/justement-python#with_streaming_response
        """
        return DocumentResourceWithStreamingResponse(self)

    def by_id(
        self,
        *,
        doc_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document by its document identifier.

        The docId is returned by the `search` endpoint as part of the result snippets.
        The response includes the full document content and metadata.

        Args:
          doc_id: The `docId` of the document that should be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/document",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"doc_id": doc_id}, document_by_id_params.DocumentByIDParams),
            ),
            cast_to=Document,
        )

    def by_ref(
        self,
        *,
        doc_ref: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document using its standard legal reference.

        This endpoint accepts Swiss legal references in their standard citation format
        and returns the corresponding document if found.

        Args:
          doc_ref: The legal reference of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/documentByRef",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"doc_ref": doc_ref}, document_by_ref_params.DocumentByRefParams),
            ),
            cast_to=Document,
        )

    def count(
        self,
        *,
        classification_facet: List[str] | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentCountResponse:
        """
        Count the number of documents matching the search criteria.

        Args:
          classification_facet: **Classification facet**: Filters results based on hierarchical categories. Each
              element in the list represents a level in the hierarchy:

              - `["Staatsrecht", "Grundrecht"]`: filters to Staatsrecht (Public Law) and
                Grundrecht (Fundamental Rights).
              - `["Privatrecht", "Zivilrecht", "Vertragsrecht"]`: filters to Privatrecht
                (Private Law), Zivilrecht (Civil Law), and Vertragsrecht (Contract Law).
              - `["Strafrecht", "Straftaten"]`: filters to Strafrecht (Criminal Law) and
                Straftaten (Offenses).

              See the full classification hierarchy:

              ```plaintext
              Staatsrecht
              ├── Abgaberecht & öffentliche Finanzen
              ├── Bau- & Planungsrecht
              ├── Bürger- & Ausländerrecht
              ├── Energie/Verkehr/Medien
              │   └── Verkehr
              ├── Grundrecht
              ├── Gesundheit & soziale Sicherheit
              ├── Öffentliches Dienstverhältnis
              ├── Ökologisches Gleichgewicht
              ├── Politische Rechte
              ├── Rechtshilfe & Auslieferung
              ├── Schuldbetreibungs- & Konkursrecht
              ├── Sozialversicherungsrecht
              │   ├── AHV
              │   ├── ALV
              │   ├── BV
              │   ├── EL
              │   ├── IV
              │   ├── KV
              │   └── UV
              ├── Unterrichtswesen & Berufsausbildung
              ├── Verfahren
              │   ├── Strafprozess
              │   ├── Zivilprozess
              │   └── Zuständigkeitsfragen
              └── Verfahrensrecht

              Privatrecht
              ├── Immaterialgüter-, Wettbewerbs- & Kartellrecht
              ├── Obligationen- & Handelsrecht
              │   ├── Gesellschaftsrecht
              │   ├── Haftpflichtrecht
              │   ├── Obligationsrecht (allgemein)
              │   └── Vertragsrecht
              └── Zivilrecht
                 ├── Erbrecht
                 ├── Familienrecht
                 ├── Personenrecht
                 └── Sachenrecht

              Strafrecht
              ├── Straf- & Massnahmenvollzug
              ├── Straftaten
              └── Strafrecht (allgemein)
              ```

          query: **Search query**: Retrieves the count of documents matching the criteria.

              - Terms are **implicitly ANDed** (e.g., `hund biss` only matches documents
                containing both terms).
              - Supports **exact phrases** (e.g., `"hund spazieren"`) and **proximity
                searches** (e.g., `"hund biss"~10`).
              - **Excludes terms** with `-term` (e.g., `beschattung -besonnung`).
              - **Prefix search** with `term*` for terms with at least 3 characters.
              - **Synonym expansion and translations** with lower relevance ranking.

              For advanced operators and tips, see
              [Justement Search Tips](https://justement.ch/en/search).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/count",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "classification_facet": classification_facet,
                        "query": query,
                    },
                    document_count_params.DocumentCountParams,
                ),
            ),
            cast_to=int,
        )

    def search(
        self,
        *,
        classification_facet: List[str] | NotGiven = NOT_GIVEN,
        language: Language | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncJustementPagination[Snippet]:
        """
        Search Justement and receive a result page with up to 10 snippets of matching
        documents, ranked by relevance.

        Args:
          classification_facet: **Classification facet**: Filters results based on hierarchical categories. Each
              element in the list represents a level in the hierarchy:

              - `["Staatsrecht", "Grundrecht"]`: filters to Staatsrecht (Public Law) and
                Grundrecht (Fundamental Rights).
              - `["Privatrecht", "Zivilrecht", "Vertragsrecht"]`: filters to Privatrecht
                (Private Law), Zivilrecht (Civil Law), and Vertragsrecht (Contract Law).
              - `["Strafrecht", "Straftaten"]`: filters to Strafrecht (Criminal Law) and
                Straftaten (Offenses).

              See the full classification hierarchy:

              ```plaintext
              Staatsrecht
              ├── Abgaberecht & öffentliche Finanzen
              ├── Bau- & Planungsrecht
              ├── Bürger- & Ausländerrecht
              ├── Energie/Verkehr/Medien
              │   └── Verkehr
              ├── Grundrecht
              ├── Gesundheit & soziale Sicherheit
              ├── Öffentliches Dienstverhältnis
              ├── Ökologisches Gleichgewicht
              ├── Politische Rechte
              ├── Rechtshilfe & Auslieferung
              ├── Schuldbetreibungs- & Konkursrecht
              ├── Sozialversicherungsrecht
              │   ├── AHV
              │   ├── ALV
              │   ├── BV
              │   ├── EL
              │   ├── IV
              │   ├── KV
              │   └── UV
              ├── Unterrichtswesen & Berufsausbildung
              ├── Verfahren
              │   ├── Strafprozess
              │   ├── Zivilprozess
              │   └── Zuständigkeitsfragen
              └── Verfahrensrecht

              Privatrecht
              ├── Immaterialgüter-, Wettbewerbs- & Kartellrecht
              ├── Obligationen- & Handelsrecht
              │   ├── Gesellschaftsrecht
              │   ├── Haftpflichtrecht
              │   ├── Obligationsrecht (allgemein)
              │   └── Vertragsrecht
              └── Zivilrecht
                 ├── Erbrecht
                 ├── Familienrecht
                 ├── Personenrecht
                 └── Sachenrecht

              Strafrecht
              ├── Straf- & Massnahmenvollzug
              ├── Straftaten
              └── Strafrecht (allgemein)
              ```

          language: Preferred language for snippets.

          page: Result page (1-based). Maximum page is total results / 10 rounded up.

          query: **Search query**: Retrieves the count of documents matching the criteria.

              - Terms are **implicitly ANDed** (e.g., `hund biss` only matches documents
                containing both terms).
              - Supports **exact phrases** (e.g., `"hund spazieren"`) and **proximity
                searches** (e.g., `"hund biss"~10`).
              - **Excludes terms** with `-term` (e.g., `beschattung -besonnung`).
              - **Prefix search** with `term*` for terms with at least 3 characters.
              - **Synonym expansion and translations** with lower relevance ranking.

              For advanced operators and tips, see
              [Justement Search Tips](https://justement.ch/en/search).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/search",
            page=SyncJustementPagination[Snippet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "classification_facet": classification_facet,
                        "language": language,
                        "page": page,
                        "query": query,
                    },
                    document_search_params.DocumentSearchParams,
                ),
            ),
            model=Snippet,
        )


class AsyncDocumentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDocumentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/justement-api/justement-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/justement-api/justement-python#with_streaming_response
        """
        return AsyncDocumentResourceWithStreamingResponse(self)

    async def by_id(
        self,
        *,
        doc_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document by its document identifier.

        The docId is returned by the `search` endpoint as part of the result snippets.
        The response includes the full document content and metadata.

        Args:
          doc_id: The `docId` of the document that should be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/document",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"doc_id": doc_id}, document_by_id_params.DocumentByIDParams),
            ),
            cast_to=Document,
        )

    async def by_ref(
        self,
        *,
        doc_ref: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Document:
        """
        Retrieve a document using its standard legal reference.

        This endpoint accepts Swiss legal references in their standard citation format
        and returns the corresponding document if found.

        Args:
          doc_ref: The legal reference of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/documentByRef",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"doc_ref": doc_ref}, document_by_ref_params.DocumentByRefParams),
            ),
            cast_to=Document,
        )

    async def count(
        self,
        *,
        classification_facet: List[str] | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentCountResponse:
        """
        Count the number of documents matching the search criteria.

        Args:
          classification_facet: **Classification facet**: Filters results based on hierarchical categories. Each
              element in the list represents a level in the hierarchy:

              - `["Staatsrecht", "Grundrecht"]`: filters to Staatsrecht (Public Law) and
                Grundrecht (Fundamental Rights).
              - `["Privatrecht", "Zivilrecht", "Vertragsrecht"]`: filters to Privatrecht
                (Private Law), Zivilrecht (Civil Law), and Vertragsrecht (Contract Law).
              - `["Strafrecht", "Straftaten"]`: filters to Strafrecht (Criminal Law) and
                Straftaten (Offenses).

              See the full classification hierarchy:

              ```plaintext
              Staatsrecht
              ├── Abgaberecht & öffentliche Finanzen
              ├── Bau- & Planungsrecht
              ├── Bürger- & Ausländerrecht
              ├── Energie/Verkehr/Medien
              │   └── Verkehr
              ├── Grundrecht
              ├── Gesundheit & soziale Sicherheit
              ├── Öffentliches Dienstverhältnis
              ├── Ökologisches Gleichgewicht
              ├── Politische Rechte
              ├── Rechtshilfe & Auslieferung
              ├── Schuldbetreibungs- & Konkursrecht
              ├── Sozialversicherungsrecht
              │   ├── AHV
              │   ├── ALV
              │   ├── BV
              │   ├── EL
              │   ├── IV
              │   ├── KV
              │   └── UV
              ├── Unterrichtswesen & Berufsausbildung
              ├── Verfahren
              │   ├── Strafprozess
              │   ├── Zivilprozess
              │   └── Zuständigkeitsfragen
              └── Verfahrensrecht

              Privatrecht
              ├── Immaterialgüter-, Wettbewerbs- & Kartellrecht
              ├── Obligationen- & Handelsrecht
              │   ├── Gesellschaftsrecht
              │   ├── Haftpflichtrecht
              │   ├── Obligationsrecht (allgemein)
              │   └── Vertragsrecht
              └── Zivilrecht
                 ├── Erbrecht
                 ├── Familienrecht
                 ├── Personenrecht
                 └── Sachenrecht

              Strafrecht
              ├── Straf- & Massnahmenvollzug
              ├── Straftaten
              └── Strafrecht (allgemein)
              ```

          query: **Search query**: Retrieves the count of documents matching the criteria.

              - Terms are **implicitly ANDed** (e.g., `hund biss` only matches documents
                containing both terms).
              - Supports **exact phrases** (e.g., `"hund spazieren"`) and **proximity
                searches** (e.g., `"hund biss"~10`).
              - **Excludes terms** with `-term` (e.g., `beschattung -besonnung`).
              - **Prefix search** with `term*` for terms with at least 3 characters.
              - **Synonym expansion and translations** with lower relevance ranking.

              For advanced operators and tips, see
              [Justement Search Tips](https://justement.ch/en/search).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/count",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "classification_facet": classification_facet,
                        "query": query,
                    },
                    document_count_params.DocumentCountParams,
                ),
            ),
            cast_to=int,
        )

    def search(
        self,
        *,
        classification_facet: List[str] | NotGiven = NOT_GIVEN,
        language: Language | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Snippet, AsyncJustementPagination[Snippet]]:
        """
        Search Justement and receive a result page with up to 10 snippets of matching
        documents, ranked by relevance.

        Args:
          classification_facet: **Classification facet**: Filters results based on hierarchical categories. Each
              element in the list represents a level in the hierarchy:

              - `["Staatsrecht", "Grundrecht"]`: filters to Staatsrecht (Public Law) and
                Grundrecht (Fundamental Rights).
              - `["Privatrecht", "Zivilrecht", "Vertragsrecht"]`: filters to Privatrecht
                (Private Law), Zivilrecht (Civil Law), and Vertragsrecht (Contract Law).
              - `["Strafrecht", "Straftaten"]`: filters to Strafrecht (Criminal Law) and
                Straftaten (Offenses).

              See the full classification hierarchy:

              ```plaintext
              Staatsrecht
              ├── Abgaberecht & öffentliche Finanzen
              ├── Bau- & Planungsrecht
              ├── Bürger- & Ausländerrecht
              ├── Energie/Verkehr/Medien
              │   └── Verkehr
              ├── Grundrecht
              ├── Gesundheit & soziale Sicherheit
              ├── Öffentliches Dienstverhältnis
              ├── Ökologisches Gleichgewicht
              ├── Politische Rechte
              ├── Rechtshilfe & Auslieferung
              ├── Schuldbetreibungs- & Konkursrecht
              ├── Sozialversicherungsrecht
              │   ├── AHV
              │   ├── ALV
              │   ├── BV
              │   ├── EL
              │   ├── IV
              │   ├── KV
              │   └── UV
              ├── Unterrichtswesen & Berufsausbildung
              ├── Verfahren
              │   ├── Strafprozess
              │   ├── Zivilprozess
              │   └── Zuständigkeitsfragen
              └── Verfahrensrecht

              Privatrecht
              ├── Immaterialgüter-, Wettbewerbs- & Kartellrecht
              ├── Obligationen- & Handelsrecht
              │   ├── Gesellschaftsrecht
              │   ├── Haftpflichtrecht
              │   ├── Obligationsrecht (allgemein)
              │   └── Vertragsrecht
              └── Zivilrecht
                 ├── Erbrecht
                 ├── Familienrecht
                 ├── Personenrecht
                 └── Sachenrecht

              Strafrecht
              ├── Straf- & Massnahmenvollzug
              ├── Straftaten
              └── Strafrecht (allgemein)
              ```

          language: Preferred language for snippets.

          page: Result page (1-based). Maximum page is total results / 10 rounded up.

          query: **Search query**: Retrieves the count of documents matching the criteria.

              - Terms are **implicitly ANDed** (e.g., `hund biss` only matches documents
                containing both terms).
              - Supports **exact phrases** (e.g., `"hund spazieren"`) and **proximity
                searches** (e.g., `"hund biss"~10`).
              - **Excludes terms** with `-term` (e.g., `beschattung -besonnung`).
              - **Prefix search** with `term*` for terms with at least 3 characters.
              - **Synonym expansion and translations** with lower relevance ranking.

              For advanced operators and tips, see
              [Justement Search Tips](https://justement.ch/en/search).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/search",
            page=AsyncJustementPagination[Snippet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "classification_facet": classification_facet,
                        "language": language,
                        "page": page,
                        "query": query,
                    },
                    document_search_params.DocumentSearchParams,
                ),
            ),
            model=Snippet,
        )


class DocumentResourceWithRawResponse:
    def __init__(self, document: DocumentResource) -> None:
        self._document = document

        self.by_id = to_raw_response_wrapper(
            document.by_id,
        )
        self.by_ref = to_raw_response_wrapper(
            document.by_ref,
        )
        self.count = to_raw_response_wrapper(
            document.count,
        )
        self.search = to_raw_response_wrapper(
            document.search,
        )


class AsyncDocumentResourceWithRawResponse:
    def __init__(self, document: AsyncDocumentResource) -> None:
        self._document = document

        self.by_id = async_to_raw_response_wrapper(
            document.by_id,
        )
        self.by_ref = async_to_raw_response_wrapper(
            document.by_ref,
        )
        self.count = async_to_raw_response_wrapper(
            document.count,
        )
        self.search = async_to_raw_response_wrapper(
            document.search,
        )


class DocumentResourceWithStreamingResponse:
    def __init__(self, document: DocumentResource) -> None:
        self._document = document

        self.by_id = to_streamed_response_wrapper(
            document.by_id,
        )
        self.by_ref = to_streamed_response_wrapper(
            document.by_ref,
        )
        self.count = to_streamed_response_wrapper(
            document.count,
        )
        self.search = to_streamed_response_wrapper(
            document.search,
        )


class AsyncDocumentResourceWithStreamingResponse:
    def __init__(self, document: AsyncDocumentResource) -> None:
        self._document = document

        self.by_id = async_to_streamed_response_wrapper(
            document.by_id,
        )
        self.by_ref = async_to_streamed_response_wrapper(
            document.by_ref,
        )
        self.count = async_to_streamed_response_wrapper(
            document.count,
        )
        self.search = async_to_streamed_response_wrapper(
            document.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from justement import Justement, AsyncJustement
from tests.utils import assert_matches_type
from justement.types import (
    Snippet,
    Document,
    DocumentCountResponse,
)
from justement.pagination import SyncJustementPagination, AsyncJustementPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocument:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_by_id(self, client: Justement) -> None:
        document = client.document.by_id(
            doc_id="docId",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_raw_response_by_id(self, client: Justement) -> None:
        response = client.document.with_raw_response.by_id(
            doc_id="docId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_streaming_response_by_id(self, client: Justement) -> None:
        with client.document.with_streaming_response.by_id(
            doc_id="docId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_by_ref(self, client: Justement) -> None:
        document = client.document.by_ref(
            doc_ref="docRef",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_raw_response_by_ref(self, client: Justement) -> None:
        response = client.document.with_raw_response.by_ref(
            doc_ref="docRef",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_streaming_response_by_ref(self, client: Justement) -> None:
        with client.document.with_streaming_response.by_ref(
            doc_ref="docRef",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_count(self, client: Justement) -> None:
        document = client.document.count()
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    def test_method_count_with_all_params(self, client: Justement) -> None:
        document = client.document.count(
            classification_facet=["string"],
            query="query",
        )
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    def test_raw_response_count(self, client: Justement) -> None:
        response = client.document.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    def test_streaming_response_count(self, client: Justement) -> None:
        with client.document.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentCountResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_search(self, client: Justement) -> None:
        document = client.document.search()
        assert_matches_type(SyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: Justement) -> None:
        document = client.document.search(
            classification_facet=["string"],
            language="de",
            page=1,
            query="query",
        )
        assert_matches_type(SyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: Justement) -> None:
        response = client.document.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(SyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: Justement) -> None:
        with client.document.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(SyncJustementPagination[Snippet], document, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDocument:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_by_id(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.by_id(
            doc_id="docId",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_raw_response_by_id(self, async_client: AsyncJustement) -> None:
        response = await async_client.document.with_raw_response.by_id(
            doc_id="docId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_streaming_response_by_id(self, async_client: AsyncJustement) -> None:
        async with async_client.document.with_streaming_response.by_id(
            doc_id="docId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_by_ref(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.by_ref(
            doc_ref="docRef",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_raw_response_by_ref(self, async_client: AsyncJustement) -> None:
        response = await async_client.document.with_raw_response.by_ref(
            doc_ref="docRef",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_streaming_response_by_ref(self, async_client: AsyncJustement) -> None:
        async with async_client.document.with_streaming_response.by_ref(
            doc_ref="docRef",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_count(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.count()
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    async def test_method_count_with_all_params(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.count(
            classification_facet=["string"],
            query="query",
        )
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    async def test_raw_response_count(self, async_client: AsyncJustement) -> None:
        response = await async_client.document.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentCountResponse, document, path=["response"])

    @parametrize
    async def test_streaming_response_count(self, async_client: AsyncJustement) -> None:
        async with async_client.document.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentCountResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_search(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.search()
        assert_matches_type(AsyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncJustement) -> None:
        document = await async_client.document.search(
            classification_facet=["string"],
            language="de",
            page=1,
            query="query",
        )
        assert_matches_type(AsyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncJustement) -> None:
        response = await async_client.document.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(AsyncJustementPagination[Snippet], document, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncJustement) -> None:
        async with async_client.document.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(AsyncJustementPagination[Snippet], document, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Snippet"]


class Snippet(BaseModel):
    doc_id: str = FieldInfo(alias="docId")

    document_url: str = FieldInfo(alias="documentUrl")

    language: str

    name: str

    snippet: str

    source: str

    year: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Language"]

Language: TypeAlias = Literal["de", "en", "fr", "it", "rm"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .snippet import Snippet as Snippet
from .document import Document as Document
from .language import Language as Language
from .snippets import Snippets as Snippets
from .document_by_id_params import DocumentByIDParams as DocumentByIDParams
from .document_count_params import DocumentCountParams as DocumentCountParams
from .document_by_ref_params import DocumentByRefParams as DocumentByRefParams
from .document_search_params import DocumentSearchParams as DocumentSearchParams
from .document_count_response import DocumentCountResponse as DocumentCountResponse
from .snippet_document_params import SnippetDocumentParams as SnippetDocumentParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .snippet import Snippet
from .._models import BaseModel

__all__ = ["Snippets"]


class Snippets(BaseModel):
    result_count: int = FieldInfo(alias="resultCount")

    results: Optional[List[Snippet]] = None
